/* global Bdf,$$ */
/**
 * Objet global définissant l'espace de nom Ficheform
 * 
 * @namespace Ficheframe,Bdf
 */
var Ficheframe = {};

Ficheframe.backgroundColor = "#f4f4f4";

Ficheframe.ARGS = {
    goto: "",
    pageResultOptions: ""
};

Ficheframe.testToolbar = function (settings) {
    var iframe = settings.iframe;
    var toolbarId = settings.toolbarId;
    var fieldeditId = settings.fieldeditId;
    var editTarget = settings.editTarget;
    var tocTarget = settings.tocTarget;
    var callback = settings.callback;
    var pageResultOptions = settings.pageResultOptions;
    var fapi = settings.fichothequeApi;
    var contentWindow = iframe.contentWindow;
    var startingLeft = settings.startingLeft;
    _showToolbar(false);
    var buttonId = "ficheframe_button_ficheedit";
    var pathName;
    try {
        pathName = contentWindow.location.pathname;
    } catch(error) {
        return;
    }
    if (!pathName.startsWith(_getFichothequeRoot())) {
        return;
    }
    var ficheTest = pathName.match(/\/fiches\/([a-z][a-z0-9]*)-([0-9]+)(-[_a-z0-9]+)?\.html$/);
    if (!ficheTest) {
        return;
    }
    var corpusName = ficheTest[1];
    var ficheId = ficheTest[2];
    var gotoManager = new Ficheframe.GotoManager(iframe, fieldeditId, startingLeft);
    fapi.loadFiche({
        corpus: corpusName,
        id: ficheId,
        callback: function (data) {
            if (data.fiche) {
                let editable = data.fiche.editable;
                gotoManager.init(editable, _updateGoto);
                let toc = _checkToc(contentWindow.document);
                let html = Bdf.render("ficheframe:toolbar", {
                    path: pathName,
                    corpusName: corpusName,
                    ficheId: ficheId,
                    editable: editable,
                    toc: toc,
                    editTarget: editTarget,
                    tocTarget: tocTarget,
                    pageResultOptions: pageResultOptions,
                    fichothequeUrl: fapi.fichothequeUrl
                });
                $$(toolbarId).html(html);
                if (html.length > 0) {
                    _showToolbar(true);
                    $(contentWindow).on('pagehide', function(e) {
                        _showToolbar(false);
                        gotoManager.hide();
                    });
                    let editLink = $$.one(buttonId);
                    let originalHref = editLink.href;
                    if (toc.length > 0) {
                        $(contentWindow).on("scroll", function(event) {
                            let threshold = contentWindow.innerHeight * 2 / 3;
                            let scrollTop = $(contentWindow.document).scrollTop();
                            let currentTocItem = false;
                            for (let i = toc.length-1; i >= 0; i--) {
                                let tocItem = toc[i];
                                if (scrollTop > $(tocItem.element).offset().top - threshold) {
                                    currentTocItem = tocItem;
                                    if (i > 0) {
                                        let previousTocItem = toc[i - 1];
                                        let $previousEl = $(previousTocItem.element);
                                        let previousBottom = $previousEl.offset().top + $previousEl.height() + (threshold / 3);
                                        if (previousBottom > scrollTop) {
                                            currentTocItem = previousTocItem;
                                        }
                                    }
                                    break;
                                }
                            }
                            $(".ficheframe-toc-Active").removeClass("ficheframe-toc-Active");
                            if (currentTocItem) {
                                $$("linkto:" + currentTocItem.id).addClass("ficheframe-toc-Active");
                                editLink.href = originalHref + "&goto=" + currentTocItem.goto;
                            } else {
                                editLink.href = originalHref;
                            }
                        });
                    }
                }
                if (callback) {
                    callback();
                }
            }
        }
    });
    
    function _updateGoto (position, top, left) {
        $$(fieldeditId).html(Bdf.render("ficheframe:fieldedit", {
            corpusName: position.checkCorpusName(corpusName),
            ficheId: position.checkFicheId(ficheId),
            editTarget: editTarget,
            goto: position.goto,
            pageResultOptions: pageResultOptions,
            fichothequeUrl: fapi.fichothequeUrl
        })).css("top", top).css("left", left);
    }
     
    function _showToolbar(display) {
        Ficheframe.show(toolbarId, display);
    }
      
    function _checkToc(document) {
        let array = new Array();
        $$(document, {bdfFormMilestone: true}).each(function (index, element) {
            let milestoneType = element.dataset.bdfFormMilestone;
            let goto;
            let $headings;
            if (milestoneType === "satellite") {
                goto = "satellite_" + element.dataset.bdfSatellite;
                $headings = $(element).children("h1");
            } else if (milestoneType === "comment") {
                goto = element.dataset.bdfComponentName;
                let $satellite = $(element).parents($$.toCssSelector({bdfFormMilestone: "satellite"}));
                if ($satellite.length > 0) {
                    goto = "satellite_" + $satellite[0].dataset.bdfSatellite + "/" + goto;
                }
                $headings = $(element).find("h2");
            }
            if ($headings) {
                $headings.each(function (index2, element2) {
                    let id = element2.id;
                    if (!id) {
                        id = Bdf.generateId();
                        element2.id = id;
                    }
                    let $element2 = $(element2);
                    let text = $element2.text();
                    if (!$element2.is(":hidden")) {
                        array.push({
                            id: id,
                            text: text,
                            element: element2,
                            goto: goto
                        });
                    }
                });
            }
        });
        return array;
    }
    
    function _getFichothequeRoot() {
        let currentPathName = window.location.pathname;
        let lastIndex = currentPathName.lastIndexOf("/");
        return currentPathName.substr(0, lastIndex + 1);
    }
};


Ficheframe.show = function (elementId, display) {
    if (display) {
        $$(elementId).removeClass("hidden");
    } else {
        $$(elementId).addClass("hidden");
    }
};